/*
 * libquassel
 * Copyright (c) 2024 Janne Mareike Koschinski
 *
 * This Source Code Form is subject to the terms of the Mozilla Public License,
 * v. 2.0. If a copy of the MPL was not distributed with this file, You can
 * obtain one at https://mozilla.org/MPL/2.0/.
 */

package de.justjanne.libquassel.protocol.api.client

import de.justjanne.libquassel.annotations.ProtocolSide
import de.justjanne.libquassel.annotations.RpcApi
import de.justjanne.libquassel.annotations.RpcParam
import de.justjanne.libquassel.annotations.RpcCall
import de.justjanne.libquassel.protocol.models.ids.BufferId
import de.justjanne.libquassel.protocol.models.ids.MsgId
import de.justjanne.libquassel.protocol.variant.QVariantList

@RpcApi("BacklogManager", side = ProtocolSide.CORE)
interface BacklogManagerClientApi {
  /**
   * Response to the corresponding [requestBacklog] call.
   * [messages] contains the messages as `QVariant<Message>`
   */
      @RpcCall("receiveBacklog")
  fun receiveBacklog(
    @RpcParam.UserType.BufferId bufferId: BufferId,
    @RpcParam.UserType.MsgId first: MsgId = MsgId(-1),
    @RpcParam.UserType.MsgId last: MsgId = MsgId(-1),
    @RpcParam.Int limit: Int = -1,
    @RpcParam.Int additional: Int = 0,
    @RpcParam.QVariantList messages: QVariantList
  )

  /**
   * Response to the corresponding [requestBacklogFiltered] call.
   * [messages] contains the messages as `QVariant<Message>`
   */
      @RpcCall("receiveBacklogFiltered")
  fun receiveBacklogFiltered(
    @RpcParam.UserType.BufferId bufferId: BufferId,
    @RpcParam.UserType.MsgId first: MsgId = MsgId(-1),
    @RpcParam.UserType.MsgId last: MsgId = MsgId(-1),
    @RpcParam.Int limit: Int = -1,
    @RpcParam.Int additional: Int = 0,
    @RpcParam.Int type: Int = -1,
    @RpcParam.Int flags: Int = -1,
    @RpcParam.QVariantList messages: QVariantList
  )

  /**
   * Response to the corresponding [requestBacklogForward] call.
   * [messages] contains the messages as `QVariant<Message>`
   */
      @RpcCall("receiveBacklogForward")
  fun receiveBacklogForward(
    @RpcParam.UserType.BufferId bufferId: BufferId,
    @RpcParam.UserType.MsgId first: MsgId = MsgId(-1),
    @RpcParam.UserType.MsgId last: MsgId = MsgId(-1),
    @RpcParam.Int limit: Int = -1,
    @RpcParam.Int type: Int = -1,
    @RpcParam.Int flags: Int = -1,
    @RpcParam.QVariantList messages: QVariantList
  )

  /**
   * Response to the corresponding [requestBacklogAll] call.
   * [messages] contains the messages as `QVariant<Message>`
   */
      @RpcCall("receiveBacklogAll")
  fun receiveBacklogAll(
    @RpcParam.UserType.MsgId first: MsgId = MsgId(-1),
    @RpcParam.UserType.MsgId last: MsgId = MsgId(-1),
    @RpcParam.Int limit: Int = -1,
    @RpcParam.Int additional: Int = 0,
    @RpcParam.QVariantList messages: QVariantList
  )

  /**
   * Response to the corresponding [requestBacklogAllFiltered] call.
   * [messages] contains the messages as `QVariant<Message>`
   */
      @RpcCall("receiveBacklogAllFiltered")
  fun receiveBacklogAllFiltered(
    @RpcParam.UserType.MsgId first: MsgId = MsgId(-1),
    @RpcParam.UserType.MsgId last: MsgId = MsgId(-1),
    @RpcParam.Int limit: Int = -1,
    @RpcParam.Int additional: Int = 0,
    @RpcParam.Int type: Int = -1,
    @RpcParam.Int flags: Int = -1,
    @RpcParam.QVariantList messages: QVariantList
  )
}
