/*
 * libquassel
 * Copyright (c) 2024 Janne Mareike Koschinski
 *
 * This Source Code Form is subject to the terms of the Mozilla Public License,
 * v. 2.0. If a copy of the MPL was not distributed with this file, You can
 * obtain one at https://mozilla.org/MPL/2.0/.
 */

package de.justjanne.libquassel.protocol.api.client


import de.justjanne.libquassel.annotations.ProtocolSide
import de.justjanne.libquassel.annotations.RpcApi
import de.justjanne.libquassel.annotations.RpcCall
import de.justjanne.libquassel.annotations.RpcParam
import de.justjanne.libquassel.protocol.models.ids.BufferId
import de.justjanne.libquassel.protocol.models.ids.NetworkId
import de.justjanne.libquassel.protocol.variant.QVariantMap

@RpcApi("BufferViewConfig", side = ProtocolSide.CORE)
interface BufferViewConfigClientApi {
  @RpcCall("addBuffer")
  fun addBuffer(@RpcParam.UserType.BufferId buffer: BufferId, @RpcParam.Int pos: Int)

  @RpcCall("moveBuffer")
  fun moveBuffer(@RpcParam.UserType.BufferId buffer: BufferId, @RpcParam.Int pos: Int)

  @RpcCall("removeBuffer")
  fun removeBuffer(@RpcParam.UserType.BufferId buffer: BufferId)

  @RpcCall("removeBufferPermanently")
  fun removeBufferPermanently(@RpcParam.UserType.BufferId buffer: BufferId)

  @RpcCall("setBufferViewName")
  fun setBufferViewName(@RpcParam.QString value: String)

  @RpcCall("setAddNewBuffersAutomatically")
  fun setAddNewBuffersAutomatically(@RpcParam.Bool value: Boolean)

  @RpcCall("setAllowedBufferTypes")
  fun setAllowedBufferTypes(@RpcParam.Int value: Int)

  @RpcCall("setDisableDecoration")
  fun setDisableDecoration(@RpcParam.Bool value: Boolean)

  @RpcCall("setHideInactiveBuffers")
  fun setHideInactiveBuffers(@RpcParam.Bool value: Boolean)

  @RpcCall("setHideInactiveNetworks")
  fun setHideInactiveNetworks(@RpcParam.Bool value: Boolean)

  @RpcCall("setMinimumActivity")
  fun setMinimumActivity(@RpcParam.Int value: Int)

  @RpcCall("setNetworkId")
  fun setNetworkId(@RpcParam.UserType.NetworkId value: NetworkId)

  @RpcCall("setShowSearch")
  fun setShowSearch(@RpcParam.Bool value: Boolean)

  @RpcCall("setSortAlphabetically")
  fun setSortAlphabetically(@RpcParam.Bool value: Boolean)

  @RpcCall("update")
  fun update(@RpcParam.QVariantMap properties: QVariantMap)
}
