/*
 * libquassel
 * Copyright (c) 2024 Janne Mareike Koschinski
 *
 * This Source Code Form is subject to the terms of the Mozilla Public License,
 * v. 2.0. If a copy of the MPL was not distributed with this file, You can
 * obtain one at https://mozilla.org/MPL/2.0/.
 */

package de.justjanne.libquassel.protocol.api.client

import de.justjanne.libquassel.annotations.ProtocolSide
import de.justjanne.libquassel.annotations.RpcApi
import de.justjanne.libquassel.annotations.RpcCall
import de.justjanne.libquassel.annotations.RpcParam
import de.justjanne.libquassel.protocol.api.ObjectName
import de.justjanne.libquassel.protocol.models.QStringList
import de.justjanne.libquassel.protocol.models.ids.IdentityId
import de.justjanne.libquassel.protocol.variant.QVariantMap

@RpcApi("Identity", side = ProtocolSide.CORE)
interface IdentityClientApi {
  @RpcCall("setAutoAwayEnabled")
  fun setAutoAwayEnabled(objectName: ObjectName, @RpcParam.Bool enabled: Boolean)


  @RpcCall("setAutoAwayReason")
  fun setAutoAwayReason(objectName: ObjectName, @RpcParam.QString reason: String?)


  @RpcCall("setAutoAwayReasonEnabled")
  fun setAutoAwayReasonEnabled(objectName: ObjectName, @RpcParam.Bool enabled: Boolean)


  @RpcCall("setAutoAwayTime")
  fun setAutoAwayTime(objectName: ObjectName, @RpcParam.Int time: Int)


  @RpcCall("setAwayNick")
  fun setAwayNick(objectName: ObjectName, @RpcParam.QString awayNick: String?)


  @RpcCall("setAwayNickEnabled")
  fun setAwayNickEnabled(objectName: ObjectName, @RpcParam.Bool enabled: Boolean)


  @RpcCall("setAwayReason")
  fun setAwayReason(objectName: ObjectName, @RpcParam.QString awayReason: String?)


  @RpcCall("setAwayReasonEnabled")
  fun setAwayReasonEnabled(objectName: ObjectName, @RpcParam.Bool enabled: Boolean)


  @RpcCall("setDetachAwayEnabled")
  fun setDetachAwayEnabled(objectName: ObjectName, @RpcParam.Bool enabled: Boolean)


  @RpcCall("setDetachAwayReason")
  fun setDetachAwayReason(objectName: ObjectName, @RpcParam.QString reason: String?)


  @RpcCall("setDetachAwayReasonEnabled")
  fun setDetachAwayReasonEnabled(objectName: ObjectName, @RpcParam.Bool enabled: Boolean)


  @RpcCall("setId")
  fun setId(objectName: ObjectName, @RpcParam.UserType.IdentityId id: IdentityId)


  @RpcCall("setIdent")
  fun setIdent(objectName: ObjectName, @RpcParam.QString ident: String?)


  @RpcCall("setIdentityName")
  fun setIdentityName(objectName: ObjectName, @RpcParam.QString name: String?)


  @RpcCall("setKickReason")
  fun setKickReason(objectName: ObjectName, @RpcParam.QString reason: String?)


  @RpcCall("setNicks")
  fun setNicks(objectName: ObjectName, @RpcParam.QStringList nicks: QStringList)


  @RpcCall("setPartReason")
  fun setPartReason(objectName: ObjectName, @RpcParam.QString reason: String?)


  @RpcCall("setQuitReason")
  fun setQuitReason(objectName: ObjectName, @RpcParam.QString reason: String?)


  @RpcCall("setRealName")
  fun setRealName(objectName: ObjectName, @RpcParam.QString realName: String?)


  @RpcCall("update")
  fun update(objectName: ObjectName, @RpcParam.QVariantMap properties: QVariantMap)
}
