/*
 * libquassel
 * Copyright (c) 2024 Janne Mareike Koschinski
 *
 * This Source Code Form is subject to the terms of the Mozilla Public License,
 * v. 2.0. If a copy of the MPL was not distributed with this file, You can
 * obtain one at https://mozilla.org/MPL/2.0/.
 */

package de.justjanne.libquassel.protocol.api.client

import de.justjanne.libquassel.annotations.ProtocolSide
import de.justjanne.libquassel.annotations.RpcApi
import de.justjanne.libquassel.annotations.RpcCall
import de.justjanne.libquassel.annotations.RpcParam
import de.justjanne.libquassel.protocol.api.ObjectName
import de.justjanne.libquassel.protocol.variant.QVariantMap
import org.threeten.bp.temporal.Temporal

@RpcApi("IrcUser", side = ProtocolSide.CORE)
interface IrcUserClientApi {
  @RpcCall("addUserModes")
  fun addUserModes(objectName: ObjectName, @RpcParam.QString modes: String)

  @RpcCall("joinChannel")
  fun joinChannel(objectName: ObjectName, @RpcParam.QString channelname: String)

  @RpcCall("partChannel")
  fun partChannel(objectName: ObjectName, @RpcParam.QString channelname: String)

  @RpcCall("quit")
  fun quit(objectName: ObjectName)

  @RpcCall("removeUserModes")
  fun removeUserModes(objectName: ObjectName, @RpcParam.QString modes: String)

  @RpcCall("setAccount")
  fun setAccount(objectName: ObjectName, @RpcParam.QString account: String)

  @RpcCall("setAway")
  fun setAway(objectName: ObjectName, @RpcParam.Bool away: Boolean)

  @RpcCall("setAwayMessage")
  fun setAwayMessage(objectName: ObjectName, @RpcParam.QString awayMessage: String)

  @RpcCall("setEncrypted")
  fun setEncrypted(objectName: ObjectName, @RpcParam.Bool encrypted: Boolean)

  @RpcCall("setHost")
  fun setHost(objectName: ObjectName, @RpcParam.QString host: String)

  @RpcCall("setIdleTime")
  fun setIdleTime(objectName: ObjectName, @RpcParam.QDateTime idleTime: Temporal)

  @RpcCall("setIrcOperator")
  fun setIrcOperator(objectName: ObjectName, @RpcParam.QString ircOperator: String)

  @RpcCall("setLastAwayMessage")
  fun setLastAwayMessage(objectName: ObjectName, @RpcParam.Int lastAwayMessage: Int)

  @RpcCall("setLastAwayMessageTime")
  fun setLastAwayMessageTime(objectName: ObjectName, @RpcParam.QDateTime lastAwayMessageTime: Temporal)

  @RpcCall("setLoginTime")
  fun setLoginTime(objectName: ObjectName, @RpcParam.QDateTime loginTime: Temporal)

  @RpcCall("setNick")
  fun setNick(objectName: ObjectName, @RpcParam.QString newNick: String)

  @RpcCall("setRealName")
  fun setRealName(objectName: ObjectName, @RpcParam.QString realName: String)

  @RpcCall("setServer")
  fun setServer(objectName: ObjectName, @RpcParam.QString server: String)

  @RpcCall("setSuserHost")
  fun setSuserHost(objectName: ObjectName, @RpcParam.QString suserHost: String)

  @RpcCall("setUser")
  fun setUser(objectName: ObjectName, @RpcParam.QString user: String)

  @RpcCall("setUserModes")
  fun setUserModes(objectName: ObjectName, @RpcParam.QString modes: String)

  @RpcCall("setWhoisServiceReply")
  fun setWhoisServiceReply(objectName: ObjectName, @RpcParam.QString whoisServiceReply: String)

  @RpcCall("updateHostmask")
  fun updateHostmask(objectName: ObjectName, @RpcParam.QString mask: String)

  @RpcCall("update")
  fun update(objectName: ObjectName, @RpcParam.QVariantMap properties: QVariantMap)
}

