/*
 * libquassel
 * Copyright (c) 2024 Janne Mareike Koschinski
 *
 * This Source Code Form is subject to the terms of the Mozilla Public License,
 * v. 2.0. If a copy of the MPL was not distributed with this file, You can
 * obtain one at https://mozilla.org/MPL/2.0/.
 */

package de.justjanne.libquassel.protocol.api.client

import de.justjanne.libquassel.annotations.ProtocolSide
import de.justjanne.libquassel.annotations.RpcApi
import de.justjanne.libquassel.annotations.RpcCall
import de.justjanne.libquassel.annotations.RpcParam
import de.justjanne.libquassel.protocol.api.ObjectName
import de.justjanne.libquassel.protocol.models.QStringList
import de.justjanne.libquassel.protocol.models.ids.IdentityId
import de.justjanne.libquassel.protocol.variant.QVariantList
import de.justjanne.libquassel.protocol.variant.QVariantMap
import java.nio.ByteBuffer

@RpcApi("Network", side = ProtocolSide.CORE)
interface NetworkClientApi {
  @RpcCall("setNetworkName")
  fun setNetworkName(objectName: ObjectName, @RpcParam.QString networkName: String)


  @RpcCall("setCurrentServer")
  fun setCurrentServer(objectName: ObjectName, @RpcParam.QString currentServer: String?)


  @RpcCall("setMyNick")
  fun setMyNick(objectName: ObjectName, @RpcParam.QString myNick: String?)


  @RpcCall("setLatency")
  fun setLatency(objectName: ObjectName, @RpcParam.Int latency: Int)


  @RpcCall("setCodecForServer")
  fun setCodecForServer(objectName: ObjectName, @RpcParam.QByteArray codecForServer: ByteBuffer)


  @RpcCall("setCodecForEncoding")
  fun setCodecForEncoding(objectName: ObjectName, @RpcParam.QByteArray codecForEncoding: ByteBuffer)


  @RpcCall("setCodecForDecoding")
  fun setCodecForDecoding(objectName: ObjectName, @RpcParam.QByteArray codecForDecoding: ByteBuffer)


  @RpcCall("setIdentity")
  fun setIdentity(objectName: ObjectName, @RpcParam.UserType.IdentityId identityId: IdentityId)


  @RpcCall("setConnected")
  fun setConnected(objectName: ObjectName, @RpcParam.Bool isConnected: Boolean)


  @RpcCall("setConnectionState")
  fun setConnectionState(objectName: ObjectName, @RpcParam.Int connectionState: Int)


  @RpcCall("setUseRandomServer")
  fun setUseRandomServer(objectName: ObjectName, @RpcParam.Bool useRandomServer: Boolean)


  @RpcCall("setPerform")
  fun setPerform(objectName: ObjectName, @RpcParam.QStringList perform: QStringList)


  @RpcCall("setSkipCaps")
  fun setSkipCaps(objectName: ObjectName, @RpcParam.QStringList skipCaps: QStringList)


  @RpcCall("setUseAutoIdentify")
  fun setUseAutoIdentify(objectName: ObjectName, @RpcParam.Bool useAutoIdentify: Boolean)


  @RpcCall("setAutoIdentifyService")
  fun setAutoIdentifyService(objectName: ObjectName, @RpcParam.QString autoIdentifyService: String)


  @RpcCall("setAutoIdentifyPassword")
  fun setAutoIdentifyPassword(objectName: ObjectName, @RpcParam.QString autoIdentifyPassword: String)


  @RpcCall("setUseSasl")
  fun setUseSasl(objectName: ObjectName, @RpcParam.Bool useSasl: Boolean)


  @RpcCall("setSaslAccount")
  fun setSaslAccount(objectName: ObjectName, @RpcParam.QString saslAccount: String)


  @RpcCall("setSaslPassword")
  fun setSaslPassword(objectName: ObjectName, @RpcParam.QString saslPassword: String)


  @RpcCall("setUseAutoReconnect")
  fun setUseAutoReconnect(objectName: ObjectName, @RpcParam.Bool useAutoReconnect: Boolean)


  @RpcCall("setAutoReconnectInterval")
  fun setAutoReconnectInterval(objectName: ObjectName, @RpcParam.UInt autoReconnectInterval: UInt)


  @RpcCall("setAutoReconnectRetries")
  fun setAutoReconnectRetries(objectName: ObjectName, @RpcParam.UShort autoReconnectRetries: UShort)


  @RpcCall("setUnlimitedReconnectRetries")
  fun setUnlimitedReconnectRetries(objectName: ObjectName, @RpcParam.Bool unlimitedReconnectRetries: Boolean)


  @RpcCall("setRejoinChannels")
  fun setRejoinChannels(objectName: ObjectName, @RpcParam.Bool rejoinChannels: Boolean)


  @RpcCall("setUseCustomMessageRate")
  fun setUseCustomMessageRate(objectName: ObjectName, @RpcParam.Bool useCustomMessageRate: Boolean)


  @RpcCall("setMessageRateBurstSize")
  fun setMessageRateBurstSize(objectName: ObjectName, @RpcParam.UInt messageRateBurstSize: UInt)


  @RpcCall("setMessageRateDelay")
  fun setMessageRateDelay(objectName: ObjectName, @RpcParam.UInt messageRateDelay: UInt)


  @RpcCall("setUnlimitedMessageRate")
  fun setUnlimitedMessageRate(objectName: ObjectName, @RpcParam.Bool unlimitedMessageRate: Boolean)


  @RpcCall("setServerList")
  fun setServerList(objectName: ObjectName, @RpcParam.QVariantList serverList: QVariantList)


  @RpcCall("addSupport")
  fun addSupport(objectName: ObjectName, @RpcParam.QString param: String, @RpcParam.QString value: String = "")


  @RpcCall("removeSupport")
  fun removeSupport(objectName: ObjectName, @RpcParam.QString param: String)


  @RpcCall("addCap")
  fun addCap(objectName: ObjectName, @RpcParam.QString capability: String, @RpcParam.QString value: String = "")


  @RpcCall("acknowledgeCap")
  fun acknowledgeCap(objectName: ObjectName, @RpcParam.QString capability: String)


  @RpcCall("removeCap")
  fun removeCap(objectName: ObjectName, @RpcParam.QString capability: String)


  @RpcCall("clearCaps")
  fun clearCaps(objectName: ObjectName)


  @RpcCall("addIrcUser")
  fun addIrcUser(objectName: ObjectName, @RpcParam.QString hostmask: String)


  @RpcCall("addIrcChannel")
  fun addIrcChannel(objectName: ObjectName, @RpcParam.QString channel: String)

  @RpcCall("update")
  fun update(objectName: ObjectName, @RpcParam.QVariantMap properties: QVariantMap)
}
