/*
 * libquassel
 * Copyright (c) 2024 Janne Mareike Koschinski
 *
 * This Source Code Form is subject to the terms of the Mozilla Public License,
 * v. 2.0. If a copy of the MPL was not distributed with this file, You can
 * obtain one at https://mozilla.org/MPL/2.0/.
 */

package de.justjanne.libquassel.protocol.api.dispatcher

import de.justjanne.libquassel.protocol.api.ObjectName
import de.justjanne.libquassel.protocol.exceptions.RpcInvocationFailedException
import de.justjanne.libquassel.protocol.variant.QVariantList
import javax.inject.Inject

class SyncHandler @Inject constructor(
  private val dispatchers: Map<String, @JvmSuppressWildcards SyncDispatcher>,
) {
  @Throws(RpcInvocationFailedException::class)
  fun invoke(className: String, objectName: ObjectName, method: String, params: QVariantList) {
    dispatchers[className]?.invoke(objectName, method, params)
      ?: throw RpcInvocationFailedException.InvokerNotFoundException(className)
  }
}
