/*
 * libquassel
 * Copyright (c) 2024 Janne Mareike Koschinski
 *
 * This Source Code Form is subject to the terms of the Mozilla Public License,
 * v. 2.0. If a copy of the MPL was not distributed with this file, You can
 * obtain one at https://mozilla.org/MPL/2.0/.
 */

package de.justjanne.libquassel.protocol.api.server

import de.justjanne.libquassel.annotations.ProtocolSide
import de.justjanne.libquassel.annotations.RpcApi
import de.justjanne.libquassel.annotations.RpcParam
import de.justjanne.libquassel.annotations.RpcCall
import de.justjanne.libquassel.protocol.models.ids.BufferId
import de.justjanne.libquassel.protocol.models.ids.MsgId
import de.justjanne.libquassel.protocol.variant.QVariantMap

@RpcApi("BufferSyncer", side = ProtocolSide.CLIENT)
interface BufferSyncerServerApi {
  @RpcCall("requestMarkBufferAsRead")
  fun requestMarkBufferAsRead(@RpcParam.UserType.BufferId buffer: BufferId)

  @RpcCall("requestMergeBuffersPermanently")
  fun requestMergeBuffersPermanently(
    @RpcParam.UserType.BufferId buffer: BufferId,
    @RpcParam.UserType.BufferId buffer2: BufferId
  )

  @RpcCall("requestRemoveBuffer")
  fun requestRemoveBuffer(@RpcParam.UserType.BufferId buffer: BufferId)

  @RpcCall("requestRenameBuffer")
  fun requestRenameBuffer(
    @RpcParam.UserType.BufferId buffer: BufferId,
    @RpcParam.QString newName: String
  )

  @RpcCall("requestSetLastSeenMsg")
  fun requestSetLastSeenMsg(
    @RpcParam.UserType.BufferId buffer: BufferId,
    @RpcParam.UserType.MsgId msgId: MsgId
  )

  @RpcCall("requestSetMarkerLine")
  fun requestSetMarkerLine(
    @RpcParam.UserType.BufferId buffer: BufferId,
    @RpcParam.UserType.MsgId msgId: MsgId
  )

  @RpcCall("requestPurgeBufferIds")
  fun requestPurgeBufferIds()

  @RpcCall("requestUpdate")
  fun requestUpdate(@RpcParam.QVariantMap properties: QVariantMap)
}
