/*
 * libquassel
 * Copyright (c) 2024 Janne Mareike Koschinski
 *
 * This Source Code Form is subject to the terms of the Mozilla Public License,
 * v. 2.0. If a copy of the MPL was not distributed with this file, You can
 * obtain one at https://mozilla.org/MPL/2.0/.
 */

package de.justjanne.libquassel.protocol.api.server

import de.justjanne.libquassel.annotations.ProtocolSide
import de.justjanne.libquassel.annotations.RpcApi
import de.justjanne.libquassel.annotations.RpcParam
import de.justjanne.libquassel.annotations.RpcCall
import de.justjanne.libquassel.protocol.api.ObjectName
import de.justjanne.libquassel.protocol.models.ids.BufferId
import de.justjanne.libquassel.protocol.variant.QVariantMap

@RpcApi("BufferViewConfig", side = ProtocolSide.CLIENT)
interface BufferViewConfigServerApi {
  @RpcCall("requestAddBuffer")
  fun requestAddBuffer(objectName: ObjectName, @RpcParam.UserType.BufferId buffer: BufferId, @RpcParam.Int pos: Int)

  @RpcCall("requestMoveBuffer")
  fun requestMoveBuffer(objectName: ObjectName, @RpcParam.UserType.BufferId buffer: BufferId, @RpcParam.Int pos: Int)

  @RpcCall("requestRemoveBuffer")
  fun requestHideBuffer(objectName: ObjectName, @RpcParam.UserType.BufferId buffer: BufferId)

  @RpcCall("requestRemoveBufferPermanently")
  fun requestRemoveBuffer(objectName: ObjectName, @RpcParam.UserType.BufferId buffer: BufferId)

  @RpcCall("requestSetBufferViewName")
  fun requestSetBufferViewName(objectName: ObjectName, @RpcParam.QString value: String)

  @RpcCall("requestUpdate")
  fun requestUpdate(objectName: ObjectName, @RpcParam.QVariantMap properties: QVariantMap)
}
