/*
 * libquassel
 * Copyright (c) 2024 Janne Mareike Koschinski
 *
 * This Source Code Form is subject to the terms of the Mozilla Public License,
 * v. 2.0. If a copy of the MPL was not distributed with this file, You can
 * obtain one at https://mozilla.org/MPL/2.0/.
 */

package de.justjanne.libquassel.protocol.api.server

import de.justjanne.libquassel.annotations.ProtocolSide
import de.justjanne.libquassel.annotations.RpcApi
import de.justjanne.libquassel.annotations.RpcCall
import de.justjanne.libquassel.annotations.RpcParam
import de.justjanne.libquassel.protocol.models.BufferInfo
import de.justjanne.libquassel.protocol.models.ids.IdentityId
import de.justjanne.libquassel.protocol.models.ids.NetworkId
import de.justjanne.libquassel.protocol.models.network.IdentityDto
import de.justjanne.libquassel.protocol.models.network.NetworkInfoDto
import de.justjanne.libquassel.protocol.variant.QVariantMap

@RpcApi(side = ProtocolSide.CLIENT)
interface RpcServerApi {
  @RpcCall(name = "2createIdentity(Identity,QVariantMap)")
  fun createIdentity(@RpcParam.UserType.Identity identity: IdentityDto, @RpcParam.QVariantMap additional: QVariantMap)

  @RpcCall(name = "2removeIdentity(IdentityId)")
  fun removeIdentity(@RpcParam.UserType.IdentityId identityId: IdentityId)

  @RpcCall(name = "2createNetwork(NetworkInfo,QStringList)")
  fun createNetwork(@RpcParam.UserType.NetworkInfo networkInfo: NetworkInfoDto, @RpcParam.QStringList channels: List<String>)

  @RpcCall(name = "2removeNetwork(NetworkId)")
  fun removeNetwork(@RpcParam.UserType.NetworkId networkId: NetworkId)

  @RpcCall(name = "2changePassword(PeerPtr,QString,QString,QString)")
  fun changePassword(@RpcParam.UserType.PeerPtr peerPtr: ULong, @RpcParam.QString user: String?, @RpcParam.QString old: String?, @RpcParam.QString new: String?)

  @RpcCall(name = "2kickClient(int)")
  fun requestKickClient(@RpcParam.Int id: Int)

  @RpcCall(name = "2sendInput(BufferInfo,QString)")
  fun sendInput(@RpcParam.UserType.BufferInfo bufferInfo: BufferInfo, @RpcParam.QString message: String?)
}
