/*
 * libquassel
 * Copyright (c) 2021 Janne Mareike Koschinski
 *
 * This Source Code Form is subject to the terms of the Mozilla Public License,
 * v. 2.0. If a copy of the MPL was not distributed with this file, You can
 * obtain one at https://mozilla.org/MPL/2.0/.
 */

package de.justjanne.libquassel.generator

import com.squareup.kotlinpoet.ClassName
import com.squareup.kotlinpoet.MemberName.Companion.member
import de.justjanne.libquassel.generator.rpcmodel.RpcModel
import de.justjanne.libquassel.generator.util.transformName

object Constants {
  fun dispatcherName(
    model: RpcModel.ObjectModel,
  ) = ClassName(
    TYPENAME_SYNCDISPATCHER.packageName,
    "${model.rpcName}${transformName(model.side.name)}Dispatcher",  )

  val TYPENAME_DAGGER_MODULE = ClassName("dagger", "Module")
  val TYPENAME_DAGGER_PROVIDES = ClassName("dagger", "Provides")
  val TYPENAME_DAGGER_BINDS = ClassName("dagger", "Binds")
  val TYPENAME_DAGGER_INTOMAP = ClassName("dagger.multibindings", "IntoMap")
  val TYPENAME_DAGGER_STRINGKEY = ClassName("dagger.multibindings", "StringKey")
  val TYPENAME_JAVAX_INJECT = ClassName("javax.inject", "Inject")
  val TYPENAME_PROXY =
    ClassName(
      "de.justjanne.libquassel.protocol.api.proxy",
      "Proxy"
    )
  val TYPENAME_SYNCDISPATCHER =
    ClassName(
      "de.justjanne.libquassel.protocol.api.dispatcher",
      "SyncDispatcher",
    )
  val TYPENAME_RPCDISPATCHER =
    ClassName(
      "de.justjanne.libquassel.protocol.api.dispatcher",
      "RpcDispatcher",
    )
  val TYPENAME_UNKNOWN_METHOD_EXCEPTION =
    ClassName(
      "de.justjanne.libquassel.protocol.exceptions",
      "RpcInvocationFailedException",
      "UnknownMethodException",
    )
  val TYPENAME_WRONG_OBJECT_TYPE_EXCEPTION =
    ClassName(
      "de.justjanne.libquassel.protocol.exceptions",
      "RpcInvocationFailedException",
      "WrongObjectTypeException",
    )
  val TYPENAME_QVARIANTLIST =
    ClassName(
      "de.justjanne.libquassel.protocol.variant",
      "QVariantList",
    )
  val TYPENAME_QVARIANT_INTOORTHROW =
    ClassName(
      "de.justjanne.libquassel.protocol.variant",
      "intoOrThrow",
    )
  val TYPENAME_OBJECTNAME =
    ClassName(
      "de.justjanne.libquassel.protocol.api",
      "ObjectName",
    )
  val MEMBERNAME_OBJECTNAME_EMPTY = TYPENAME_OBJECTNAME.nestedClass("Companion").member("EMPTY")
  val TYPENAME_RPCPARAM = ClassName("de.justjanne.libquassel.annotations", "RpcParam")
  val TYPENAME_QTTYPE = ClassName("de.justjanne.libquassel.protocol.models.types", "QtType")
  val TYPENAME_QUASSELTYPE = ClassName("de.justjanne.libquassel.protocol.models.types", "QuasselType")
  val TYPENAME_GENERATED =
    ClassName(
      "de.justjanne.libquassel.annotations",
      "Generated",
    )

  init {
    System.setProperty("idea.io.use.nio2", "true")
  }
}
