/*
 * libquassel
 * Copyright (c) 2021 Janne Mareike Koschinski
 *
 * This Source Code Form is subject to the terms of the Mozilla Public License,
 * v. 2.0. If a copy of the MPL was not distributed with this file, You can
 * obtain one at https://mozilla.org/MPL/2.0/.
 */

package de.justjanne.libquassel.generator

import com.squareup.kotlinpoet.AnnotationSpec
import com.squareup.kotlinpoet.ClassName
import com.squareup.kotlinpoet.FileSpec
import com.squareup.kotlinpoet.FunSpec
import com.squareup.kotlinpoet.KModifier
import com.squareup.kotlinpoet.TypeSpec
import de.justjanne.libquassel.annotations.ProtocolSide
import de.justjanne.libquassel.generator.kotlinmodel.KotlinModel
import de.justjanne.libquassel.generator.rpcmodel.RpcModel
import de.justjanne.libquassel.generator.util.transformName
import de.justjanne.libquassel.generator.visitors.DispatcherGenerator

object DispatcherModuleGenerator {
  fun generateModule(side: ProtocolSide, objects: List<RpcModel.ObjectModel>): KotlinModel.FileModel? {
    if (objects.isEmpty()) return null
    val name =
      ClassName(
        Constants.TYPENAME_SYNCDISPATCHER.packageName,
        "${transformName(side.name)}DispatcherModule",
      )
    return KotlinModel.FileModel(
      objects.map(RpcModel.ObjectModel::source),
      FileSpec.builder(name.packageName, name.simpleName)
        .addType(
          TypeSpec.interfaceBuilder(name)
            .addAnnotation(Constants.TYPENAME_DAGGER_MODULE)
            .addFunctions(
              objects
                .filter { it.side != side }
                .map {
                  if (it.rpcName.isEmpty()) {
                    val className = DispatcherGenerator.toClassName(it)
                    FunSpec.builder("provide${Constants.dispatcherName(it).simpleName}")
                      .addModifiers(KModifier.ABSTRACT)
                      .addParameter("impl", className)
                      .returns(Constants.TYPENAME_RPCDISPATCHER)
                      .addAnnotation(Constants.TYPENAME_DAGGER_BINDS)
                      .build()
                  } else {
                    val className = DispatcherGenerator.toClassName(it)
                    FunSpec.builder("provide${Constants.dispatcherName(it).simpleName}")
                      .addModifiers(KModifier.ABSTRACT)
                      .addParameter("impl", className)
                      .returns(Constants.TYPENAME_SYNCDISPATCHER)
                      .addAnnotation(Constants.TYPENAME_DAGGER_BINDS)
                      .addAnnotation(Constants.TYPENAME_DAGGER_INTOMAP)
                      .addAnnotation(
                        AnnotationSpec.builder(Constants.TYPENAME_DAGGER_STRINGKEY).addMember("%S", it.rpcName).build()
                      )
                      .build()
                  }
                }
            )
            .build(),
        ).build(),
    )
  }
}
