/*
 * libquassel
 * Copyright (c) 2021 Janne Mareike Koschinski
 *
 * This Source Code Form is subject to the terms of the Mozilla Public License,
 * v. 2.0. If a copy of the MPL was not distributed with this file, You can
 * obtain one at https://mozilla.org/MPL/2.0/.
 */

package de.justjanne.libquassel.generator

import com.google.devtools.ksp.processing.CodeGenerator
import com.google.devtools.ksp.processing.KSPLogger
import com.google.devtools.ksp.processing.Resolver
import com.google.devtools.ksp.processing.SymbolProcessor
import com.google.devtools.ksp.symbol.KSAnnotated
import de.justjanne.libquassel.annotations.ProtocolSide
import de.justjanne.libquassel.annotations.RpcApi
import de.justjanne.libquassel.generator.visitors.KotlinModelGenerator
import de.justjanne.libquassel.generator.visitors.KotlinSaver
import de.justjanne.libquassel.generator.visitors.DispatcherGenerator
import de.justjanne.libquassel.generator.visitors.ProxyGenerator
import de.justjanne.libquassel.generator.visitors.RpcModelCollector

class InvokerProcessor(
  private val codeGenerator: CodeGenerator,
  private val logger: KSPLogger,
) : SymbolProcessor {
  override fun process(resolver: Resolver): List<KSAnnotated> {
    val annotationModels = resolver.getSymbolsWithAnnotation(RpcApi::class.java.canonicalName)
    val rpcModels = annotationModels.mapNotNull { it.accept(KotlinModelGenerator(resolver, logger), Unit) }
    val rpcObjects = RpcModelCollector().apply {
      rpcModels.forEach { it.accept(this) }
    }.objects

    rpcModels.flatMap { listOf(it.accept(DispatcherGenerator()), it.accept(ProxyGenerator())) }
      .plusElement(DispatcherModuleGenerator.generateModule(ProtocolSide.CLIENT, rpcObjects))
      .plusElement(DispatcherModuleGenerator.generateModule(ProtocolSide.CORE, rpcObjects))
      .plusElement(ProxyModuleGenerator.generateModule(ProtocolSide.CLIENT, rpcObjects))
      .plusElement(ProxyModuleGenerator.generateModule(ProtocolSide.CORE, rpcObjects))
      .filterNotNull()
      .forEach { it.accept(KotlinSaver(), codeGenerator) }

    return emptyList()
  }
}
