/*
 * libquassel
 * Copyright (c) 2021 Janne Mareike Koschinski
 *
 * This Source Code Form is subject to the terms of the Mozilla Public License,
 * v. 2.0. If a copy of the MPL was not distributed with this file, You can
 * obtain one at https://mozilla.org/MPL/2.0/.
 */

package de.justjanne.libquassel.generator

import com.squareup.kotlinpoet.ClassName
import com.squareup.kotlinpoet.FileSpec
import com.squareup.kotlinpoet.FunSpec
import com.squareup.kotlinpoet.KModifier
import com.squareup.kotlinpoet.TypeSpec
import de.justjanne.libquassel.annotations.ProtocolSide
import de.justjanne.libquassel.generator.kotlinmodel.KotlinModel
import de.justjanne.libquassel.generator.rpcmodel.RpcModel
import de.justjanne.libquassel.generator.util.transformName
import de.justjanne.libquassel.generator.visitors.ProxyGenerator

object ProxyModuleGenerator {
  fun generateModule(side: ProtocolSide, objects: List<RpcModel.ObjectModel>): KotlinModel.FileModel? {
    if (objects.isEmpty()) return null
    val name =
      ClassName(
        Constants.TYPENAME_PROXY.packageName,
        "${transformName(side.name)}ProxyModule",
      )
    return KotlinModel.FileModel(
      objects.map(RpcModel.ObjectModel::source),
      FileSpec.builder(name.packageName, name.simpleName)
        .addType(
          TypeSpec.interfaceBuilder(name)
            .addAnnotation(Constants.TYPENAME_DAGGER_MODULE)
            .addFunctions(
              objects
                .filter { it.side == side }
                .map {
                  val className = ProxyGenerator.toClassName(it)
                  FunSpec.builder("provide${Constants.dispatcherName(it).simpleName}")
                    .addModifiers(KModifier.ABSTRACT)
                    .addParameter("impl", className)
                    .returns(it.name)
                    .addAnnotation(Constants.TYPENAME_DAGGER_BINDS)
                    .build()
                }
            )
            .build(),
        ).build(),
    )
  }
}
