/*
 * libquassel
 * Copyright (c) 2021 Janne Mareike Koschinski
 *
 * This Source Code Form is subject to the terms of the Mozilla Public License,
 * v. 2.0. If a copy of the MPL was not distributed with this file, You can
 * obtain one at https://mozilla.org/MPL/2.0/.
 */

package de.justjanne.libquassel.generator.annotation

import com.google.devtools.ksp.processing.Resolver
import com.google.devtools.ksp.symbol.KSAnnotated
import de.justjanne.libquassel.annotations.RpcCall
import de.justjanne.libquassel.generator.util.ksp.findAnnotationWithType
import de.justjanne.libquassel.generator.util.ksp.getMember

data class RpcFunctionAnnotation(
  val name: String?,
  val type: Type
) {
  companion object {
    fun of(
      it: KSAnnotated,
      resolver: Resolver,
    ): RpcFunctionAnnotation? =
      it.findAnnotationWithType<RpcCall>(resolver)?.let { annotation ->
        RpcFunctionAnnotation(
          name = annotation.getMember<String>("name")?.ifBlank { null },
          type = Type.RPC,
        )
      } ?: it.findAnnotationWithType<RpcCall>(resolver)?.let { annotation ->
        RpcFunctionAnnotation(
          name = annotation.getMember<String>("name")?.ifBlank { null },
          type = Type.SYNC,
        )
      }
  }

  enum class Type {
    RPC,
    SYNC
  }
}
