/*
 * libquassel
 * Copyright (c) 2024 Janne Mareike Koschinski
 *
 * This Source Code Form is subject to the terms of the Mozilla Public License,
 * v. 2.0. If a copy of the MPL was not distributed with this file, You can
 * obtain one at https://mozilla.org/MPL/2.0/.
 */

package de.justjanne.libquassel.generator.annotation

import com.google.devtools.ksp.symbol.KSAnnotated
import com.squareup.kotlinpoet.ClassName
import com.squareup.kotlinpoet.MemberName
import com.squareup.kotlinpoet.MemberName.Companion.member
import de.justjanne.libquassel.generator.Constants
import de.justjanne.libquassel.generator.util.ksp.asClassName

data class RpcParameterAnnotation(
  val type: MemberName
) {
  companion object {
    private fun remapTypes(type: ClassName): ClassName? = when (type) {
      Constants.TYPENAME_RPCPARAM.nestedClass("UserType") -> Constants.TYPENAME_QUASSELTYPE
      Constants.TYPENAME_RPCPARAM -> Constants.TYPENAME_QTTYPE
      else -> type.enclosingClassName()?.let {
        remapTypes(it)?.nestedClass(type.simpleName)
      }
    }

    fun of(
      it: KSAnnotated,
    ): RpcParameterAnnotation? =
      it.annotations.mapNotNull {
        val annotationType = it.annotationType.resolve().asClassName()
        annotationType.enclosingClassName()
          ?.let { remapTypes(it) }
          ?.let { RpcParameterAnnotation(it.member(annotationType.simpleName)) }
      }.firstOrNull()
  }
}
