/*
 * libquassel
 * Copyright (c) 2021 Janne Mareike Koschinski
 *
 * This Source Code Form is subject to the terms of the Mozilla Public License,
 * v. 2.0. If a copy of the MPL was not distributed with this file, You can
 * obtain one at https://mozilla.org/MPL/2.0/.
 */

package de.justjanne.libquassel.generator.rpcmodel

import com.google.devtools.ksp.symbol.KSClassDeclaration
import com.google.devtools.ksp.symbol.KSFunctionDeclaration
import com.google.devtools.ksp.symbol.KSValueParameter
import com.squareup.kotlinpoet.ClassName
import com.squareup.kotlinpoet.MemberName
import com.squareup.kotlinpoet.TypeName
import de.justjanne.libquassel.annotations.ProtocolSide
import de.justjanne.libquassel.generator.annotation.RpcFunctionAnnotation

sealed class RpcModel {
  data class ObjectModel(
    val source: KSClassDeclaration,
    val name: ClassName,
    val rpcName: String,
    val side: ProtocolSide,
    val methods: List<FunctionModel>,
  ) : RpcModel() {
    override fun <R> accept(
      visitor: RpcModelVisitor<R>,
    ) = visitor.visitObjectModel(this)
  }

  data class FunctionModel(
    val source: KSFunctionDeclaration,
    val static: Boolean,
    val name: String,
    val rpcName: String?,
    val type: RpcFunctionAnnotation.Type,
    val parameters: List<ParameterModel>,
  ) : RpcModel() {
    override fun <R> accept(
      visitor: RpcModelVisitor<R>,
    ) = visitor.visitFunctionModel(this)
  }

  data class ParameterModel(
    val source: KSValueParameter,
    val name: String,
    val type: TypeName,
    val rpcType: MemberName?
  ) : RpcModel() {
    override fun <R> accept(
      visitor: RpcModelVisitor<R>,
    ) = visitor.visitParameterModel(this)
  }

  abstract fun <R> accept(
    visitor: RpcModelVisitor<R>,
  ): R
}
