/*
 * Copyright (C) 2021 Zac Sweers
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package de.justjanne.libquassel.generator.util.ksp

import com.google.devtools.ksp.symbol.KSDeclaration
import com.google.devtools.ksp.symbol.KSType
import com.google.devtools.ksp.symbol.KSTypeAlias
import com.google.devtools.ksp.symbol.KSTypeReference
import com.google.devtools.ksp.symbol.Variance
import com.squareup.kotlinpoet.ClassName
import com.squareup.kotlinpoet.ParameterizedTypeName.Companion.parameterizedBy
import com.squareup.kotlinpoet.TypeName
import com.squareup.kotlinpoet.WildcardTypeName

fun KSDeclaration.asTypeName(): TypeName = ClassName(packageName.asString(), simpleName.asString())

fun KSTypeReference.asTypeName(): TypeName = resolve().asTypeName()

fun KSType.asTypeName(): TypeName {
  when (val decl = declaration) {
    is KSTypeAlias -> return decl.type.resolve().asTypeName()
      .copy(nullable = isMarkedNullable)
  }

  val baseType = asClassName()
  if (arguments.isEmpty()) {
    return baseType
      .copy(nullable = isMarkedNullable)
  }

  val parameters =
    arguments.map {
      val type = it.type?.resolve()
      when (it.variance) {
        Variance.STAR ->
          WildcardTypeName.producerOf(Any::class)
            .copy(nullable = true)
        Variance.INVARIANT ->
          type!!.asTypeName()
            .copy(nullable = type.isMarkedNullable)
        Variance.COVARIANT ->
          WildcardTypeName.producerOf(type!!.asTypeName())
            .copy(nullable = type.isMarkedNullable)
        Variance.CONTRAVARIANT ->
          WildcardTypeName.consumerOf(type!!.asTypeName())
            .copy(nullable = type.isMarkedNullable)
      }
    }

  return baseType.parameterizedBy(parameters)
    .copy(nullable = isMarkedNullable)
}
