/*
 * Copyright (C) 2021 Zac Sweers
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package de.justjanne.libquassel.generator.util.ksp

import com.google.devtools.ksp.symbol.KSType
import com.squareup.kotlinpoet.ClassName
import com.squareup.kotlinpoet.asClassName
import kotlin.reflect.KClass

internal inline fun <reified T : Enum<T>> KSType.toEnum(): T? {
  return asClassName().toEnum(T::class)
}

internal inline fun <reified T : Enum<T>> ClassName.toEnum(): T? {
  return toEnum(T::class)
}

internal fun <T : Enum<T>> ClassName.toEnum(clazz: KClass<T>): T? {
  val enumClassName = clazz.asClassName()
  return clazz.java.enumConstants.find {
    this.canonicalName == enumClassName.nestedClass(it.name).canonicalName
  }
}
