/*
 * libquassel
 * Copyright (c) 2021 Janne Mareike Koschinski
 *
 * This Source Code Form is subject to the terms of the Mozilla Public License,
 * v. 2.0. If a copy of the MPL was not distributed with this file, You can
 * obtain one at https://mozilla.org/MPL/2.0/.
 */

package de.justjanne.libquassel.protocol.serializers.quassel

import de.justjanne.libquassel.protocol.features.FeatureSet
import de.justjanne.libquassel.protocol.io.ChainedByteBuffer
import de.justjanne.libquassel.protocol.models.ids.NetworkId
import de.justjanne.libquassel.protocol.serializers.PrimitiveSerializer
import de.justjanne.libquassel.protocol.serializers.qt.IntSerializer
import java.nio.ByteBuffer

/**
 * Serializer for [NetworkId]
 */
object NetworkIdSerializer : PrimitiveSerializer<NetworkId> {
  override val javaType: Class<out NetworkId> = NetworkId::class.java

  override fun serialize(
    buffer: ChainedByteBuffer,
    data: NetworkId,
    featureSet: FeatureSet,
  ) {
    IntSerializer.serialize(buffer, data.id, featureSet)
  }

  override fun deserialize(
    buffer: ByteBuffer,
    featureSet: FeatureSet,
  ): NetworkId {
    return NetworkId(IntSerializer.deserialize(buffer, featureSet))
  }
}
